@@ -16,9 +16,9 @@
 #ifndef MUSICBRAINZQUERY_H
 #define MUSICBRAINZQUERY_H
 
-#include <config.h>
+#include <config-juk.h>
 
-#if HAVE_MUSICBRAINZ
+#ifdef HAVE_TUNEPIMP
 
 #include "ktrm.h"
 #include "filehandle.h"
@@ -41,3 +41,5 @@ private:
 
 #endif
 #endif
+
+// vim: set et sw=4 tw=0 sta:
