@@ -13,23 +13,36 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "nowplaying.h"
+
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kdebug.h>
-#include <kurldrag.h>
+#include <krandom.h>
+#include <kglobalsettings.h>
 #include <kio/netaccess.h>
 
-#include <qimage.h>
-#include <qlayout.h>
-#include <qevent.h>
-#include <qdragobject.h>
-#include <qimage.h>
-#include <qtimer.h>
+#include <QImage>
+#include <QLayout>
+#include <QEvent>
+#include <QDrag>
+#include <QTimer>
+#include <QPoint>
+#include <QFrame>
+#include <QDropEvent>
+#include <QLabel>
+#include <QVBoxLayout>
+#include <QDragEnterEvent>
+#include <QMouseEvent>
+#include <QUrl>
+#include <QList>
+#include <QTextDocument>
 
-#include "nowplaying.h"
 #include "playlistcollection.h"
+#include "playlistitem.h"
 #include "playermanager.h"
 #include "coverinfo.h"
+#include "covermanager.h"
 #include "tag.h"
 #include "playlistitem.h"
 #include "collectionlist.h"
@@ -46,24 +59,26 @@ struct Line : public QFrame
 // NowPlaying
 ////////////////////////////////////////////////////////////////////////////////
 
-NowPlaying::NowPlaying(QWidget *parent, PlaylistCollection *collection, const char *name) :
-    QHBox(parent, name),
+NowPlaying::NowPlaying(QWidget *parent, PlaylistCollection *collection) :
+    QWidget(parent),
     m_observer(this, collection),
+    // Also watch the collection
+    m_collectionListObserver(this, CollectionList::instance()),
     m_collection(collection)
 {
-    // m_observer is set to watch the PlaylistCollection, also watch for
-    // changes that come from CollectionList.
+    setObjectName("NowPlaying");
 
-    CollectionList::instance()->addObserver(&m_observer);
+    QHBoxLayout *layout = new QHBoxLayout(this);
+    setLayout(layout);
 
-    layout()->setMargin(5);
-    layout()->setSpacing(3);
-    setFixedHeight(imageSize + 2 + layout()->margin() * 2);
+    layout->setMargin(0);
+    layout->setSpacing(3);
+    setFixedHeight(imageSize + 2);
 
-    setStretchFactor(new CoverItem(this), 0);
-    setStretchFactor(new TrackItem(this), 2);
-    setStretchFactor(new Line(this), 0);
-    setStretchFactor(new HistoryItem(this), 1);
+    layout->addWidget(new CoverItem(this), 0);
+    layout->addWidget(new TrackItem(this), 2);
+    layout->addWidget(new Line(this), 0);
+    layout->addWidget(new HistoryItem(this), 1);
 
     connect(PlayerManager::instance(), SIGNAL(signalPlay()), this, SLOT(slotUpdate()));
     connect(PlayerManager::instance(), SIGNAL(signalStop()), this, SLOT(slotUpdate()));
@@ -92,11 +107,8 @@ void NowPlaying::slotUpdate()
     else
         show();
 
-    for(QValueList<NowPlayingItem *>::Iterator it = m_items.begin();
-        it != m_items.end(); ++it)
-    {
-        (*it)->update(file);
-    }
+    foreach(NowPlayingItem *item, m_items)
+        item->update(file);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -104,12 +116,11 @@ void NowPlaying::slotUpdate()
 ////////////////////////////////////////////////////////////////////////////////
 
 CoverItem::CoverItem(NowPlaying *parent) :
-    QLabel(parent, "CoverItem"),
+    QLabel(parent),
     NowPlayingItem(parent)
 {
+    setObjectName("CoverItem");
     setFixedHeight(parent->height() - parent->layout()->margin() * 2);
-    setFrameStyle(Box | Plain);
-    setLineWidth(1);
     setMargin(1);
     setAcceptDrops(true);
 }
@@ -120,8 +131,9 @@ void CoverItem::update(const FileHandle &file)
 
     if(file.coverInfo()->hasCover()) {
         show();
-        QImage image = file.coverInfo()->pixmap(CoverInfo::FullSize).convertToImage();
-        setPixmap(image.smoothScale(imageSize, imageSize, QImage::ScaleMax));
+        setPixmap(
+	    file.coverInfo()->pixmap(CoverInfo::Thumbnail)
+	    .scaled(imageSize, imageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation));
     }
     else
         hide();
@@ -129,32 +141,76 @@ void CoverItem::update(const FileHandle &file)
 
 void CoverItem::mouseReleaseEvent(QMouseEvent *event)
 {
+    if(m_dragging) {
+        m_dragging = false;
+        return;
+    }
+
     if(event->x() >= 0 && event->y() >= 0 &&
        event->x() < width() && event->y() < height() &&
-       event->button() == LeftButton &&
+       event->button() == Qt::LeftButton &&
        m_file.coverInfo()->hasCover())
     {
         m_file.coverInfo()->popup();
     }
-    
+
     QLabel::mousePressEvent(event);
 }
 
+void CoverItem::mousePressEvent(QMouseEvent *e)
+{
+    m_dragging = false;
+    m_dragStart = e->globalPos();
+}
+
+void CoverItem::mouseMoveEvent(QMouseEvent *e)
+{
+    if(m_dragging)
+        return;
+
+    QPoint diff = m_dragStart - e->globalPos();
+    if(diff.manhattanLength() > KGlobalSettings::dndEventDelay()) {
+
+        // Start a drag.
+
+        m_dragging = true;
+
+        QDrag *drag = new QDrag(this);
+        CoverDrag *data = new CoverDrag(m_file.coverInfo()->coverId());
+
+        drag->setMimeData(data);
+        drag->exec(Qt::CopyAction);
+    }
+}
+
 void CoverItem::dragEnterEvent(QDragEnterEvent *e)
 {
-    e->accept(QImageDrag::canDecode(e) || KURLDrag::canDecode(e));
+    e->setAccepted(CoverDrag::isCover(e->mimeData()) || e->mimeData()->hasUrls());
 }
 
 void CoverItem::dropEvent(QDropEvent *e)
 {
     QImage image;
-    KURL::List urls;
+    QList<QUrl> urls;
+    coverKey key;
 
-    if(QImageDrag::decode(e, image)) {
-        m_file.coverInfo()->setCover(image);
+    if(e->source() == this)
+        return;
+
+    key = CoverDrag::idFromData(e->mimeData());
+    if(key != CoverManager::NoMatch) {
+        m_file.coverInfo()->setCoverId(key);
         update(m_file);
     }
-    else if(KURLDrag::decode(e, urls)) {
+    else if(e->mimeData()->hasImage()) {
+        m_file.coverInfo()->setCover(qvariant_cast<QImage>(e->mimeData()->imageData()));
+        update(m_file);
+    }
+    else {
+        urls = e->mimeData()->urls();
+        if(urls.isEmpty())
+            return;
+
         QString fileName;
 
         if(KIO::NetAccess::download(urls.front(), fileName, this)) {
@@ -163,12 +219,12 @@ void CoverItem::dropEvent(QDropEvent *e)
                 update(m_file);
             }
             else
-                kdError(65432) << "Unable to load image from " << urls.front() << endl;
+                kError(65432) << "Unable to load image from " << urls.front() << endl;
 
             KIO::NetAccess::removeTempFile(fileName);
         }
         else
-            kdError(65432) << "Unable to download " << urls.front() << endl;
+            kError(65432) << "Unable to download " << urls.front() << endl;
     }
 }
 
@@ -177,22 +233,30 @@ void CoverItem::dropEvent(QDropEvent *e)
 ////////////////////////////////////////////////////////////////////////////////
 
 TrackItem::TrackItem(NowPlaying *parent) :
-    QWidget(parent, "TrackItem"),
+    QWidget(parent),
     NowPlayingItem(parent)
 {
+    setObjectName("TrackItem");
     setFixedHeight(parent->height() - parent->layout()->margin() * 2);
     setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
 
     QVBoxLayout *layout = new QVBoxLayout(this);
 
-    m_label = new LinkLabel(this);
-    m_label->setLinkUnderline(false);
+    m_label = new QLabel(this);
+    m_label->setWordWrap(true);
+
+    m_label->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::LinksAccessibleByKeyboard);
+
+#ifdef __GNUC__
+    #warning We should potentially check on URL underlining.
+#endif
+    /* m_label->setLinkUnderline(false); */
 
     layout->addStretch();
     layout->addWidget(m_label);
     layout->addStretch();
 
-    connect(m_label, SIGNAL(linkClicked(const QString &)), this,
+    connect(m_label, SIGNAL(linkActivated(const QString &)), this,
             SLOT(slotOpenLink(const QString &)));
 }
 
@@ -218,10 +282,10 @@ void TrackItem::slotOpenLink(const QString &link)
 
 void TrackItem::slotUpdate()
 {
-    QString title  = QStyleSheet::escape(m_file.tag()->title());
-    QString artist = QStyleSheet::escape(m_file.tag()->artist());
-    QString album  = QStyleSheet::escape(m_file.tag()->album());
-    QString separator = (artist.isNull() || album.isNull()) ? QString::null : QString(" - ");
+    QString title  = Qt::escape(m_file.tag()->title());
+    QString artist = Qt::escape(m_file.tag()->artist());
+    QString album  = Qt::escape(m_file.tag()->album());
+    QString separator = (artist.isNull() || album.isNull()) ? QString::null : QString(" - ");	//krazy:exclude=nullstrassign for old broken gcc
 
     // This block-o-nastiness makes the font smaller and smaller until it actually fits.
 
@@ -242,7 +306,7 @@ void TrackItem::slotUpdate()
         --size;
     } while(m_label->heightForWidth(m_label->width()) > imageSize && size >= 0);
 
-    m_label->setFixedHeight(QMIN(imageSize, m_label->heightForWidth(m_label->width())));
+    m_label->setFixedHeight(qMin(imageSize, m_label->heightForWidth(m_label->width())));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -250,12 +314,12 @@ void TrackItem::slotUpdate()
 ////////////////////////////////////////////////////////////////////////////////
 
 HistoryItem::HistoryItem(NowPlaying *parent) :
-    LinkLabel(parent, "HistoryItem"),
+    QLabel(parent),
     NowPlayingItem(parent)
 {
     setFixedHeight(parent->height() - parent->layout()->margin() * 2);
     setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
-    setLinkUnderline(false);
+    /* setLinkUnderline(false); */
     setText(QString("<b>%1</b>").arg(i18n("History")));
 
     m_timer = new QTimer(this);
@@ -268,17 +332,15 @@ void HistoryItem::update(const FileHandle &file)
         return;
 
     if(m_history.count() >= 10)
-        m_history.remove(m_history.fromLast());
+        m_history.removeLast();
 
     QString format = "<br /><a href=\"%1\"><font size=\"-1\">%2</font></a>";
     QString current = QString("<b>%1</b>").arg(i18n("History"));
     QString previous;
 
-    for(QValueList<Item>::ConstIterator it = m_history.begin();
-        it != m_history.end(); ++it)
-    {
+    foreach(const Item &historyItem, m_history) {
         previous = current;
-        current.append(format.arg((*it).anchor).arg(QStyleSheet::escape((*it).file.tag()->title())));
+        current.append(format.arg(historyItem.anchor).arg(Qt::escape(historyItem.file.tag()->title())));
         setText(current);
         if(heightForWidth(width()) > imageSize) {
             setText(previous);
@@ -288,23 +350,22 @@ void HistoryItem::update(const FileHandle &file)
 
     m_file = file;
     m_timer->stop();
-    m_timer->start(HistoryPlaylist::delay(), true);
+    m_timer->setSingleShot(true);
+    m_timer->start(HistoryPlaylist::delay());
 }
 
 void HistoryItem::openLink(const QString &link)
 {
-    for(QValueList<Item>::ConstIterator it = m_history.begin();
-        it != m_history.end(); ++it)
-    {
-        if((*it).anchor == link) {
-            if((*it).playlist) {
-                CollectionListItem *collectionItem = 
-                    CollectionList::instance()->lookup((*it).file.absFilePath());
-                PlaylistItem *item = collectionItem->itemForPlaylist((*it).playlist);
-                (*it).playlist->clearSelection();
-                (*it).playlist->setSelected(item, true);
-                (*it).playlist->ensureItemVisible(item);
-                NowPlayingItem::parent()->collection()->raise((*it).playlist);
+    foreach(const Item &historyItem, m_history) {
+        if(historyItem.anchor == link) {
+            if(historyItem.playlist) {
+                CollectionListItem *collectionItem =
+                    CollectionList::instance()->lookup(historyItem.file.absFilePath());
+                PlaylistItem *item = collectionItem->itemForPlaylist(historyItem.playlist);
+                historyItem.playlist->clearSelection();
+                historyItem.playlist->setSelected(item, true);
+                historyItem.playlist->ensureItemVisible(item);
+                NowPlayingItem::parent()->collection()->raise(historyItem.playlist);
             }
             break;
         }
@@ -318,7 +379,7 @@ void HistoryItem::slotAddPlaying()
     PlayerManager *manager = PlayerManager::instance();
 
     if(manager->playing() && manager->playingFile() == m_file) {
-        m_history.prepend(Item(KApplication::randomString(20),
+        m_history.prepend(Item(KRandom::randomString(20),
                                m_file, Playlist::playingItem()->playlist()));
     }
 
@@ -327,4 +388,4 @@ void HistoryItem::slotAddPlaying()
 
 #include "nowplaying.moc"
 
-// vim: set et sw=4 ts=8:
+// vim: set et sw=4 tw=0 sta:
