@@ -15,38 +15,41 @@
 #ifndef PLAYLIST_COLLECTION_H
 #define PLAYLIST_COLLECTION_H
 
-#include "playlistinterface.h"
 #include "stringhash.h"
-#include "jukIface.h"
+#include "playlistinterface.h"
 
 #include <kshortcut.h>
 #include <klocale.h>
 #include <kdirlister.h>
 
-#include <qguardedptr.h>
+#include <QPointer>
+
+class QPixmap;
+class QStackedWidget;
 
-class QWidgetStack;
 class KAction;
-class Playlist;
-class PlaylistItem;
+
 class HistoryPlaylist;
 class UpcomingPlaylist;
 class SearchPlaylist;
 class DynamicPlaylist;
+class PlaylistItem;
+class Playlist;
 
-typedef QValueList<Playlist *> PlaylistList;
-typedef QValueList<PlaylistItem *> PlaylistItemList;
+template<class T>
+class QList;
 
-class PlaylistCollection : public PlaylistInterface, CollectionIface
+typedef QList<PlaylistItem *> PlaylistItemList;
+typedef QList<Playlist *> PlaylistList;
+
+class PlaylistCollection : public PlaylistInterface
 {
     friend class Playlist;
     friend class CollectionList;
     friend class DynamicPlaylist;
 
-    static PlaylistCollection *m_instance;
-
 public:
-    PlaylistCollection(QWidgetStack *playlistStack);
+    PlaylistCollection(QStackedWidget *playlistStack);
     virtual ~PlaylistCollection();
 
     static PlaylistCollection *instance() { return m_instance; }
@@ -66,7 +69,7 @@ public:
     virtual QStringList playlists() const;
     virtual void createPlaylist(const QString &name);
     virtual void createDynamicPlaylist(const PlaylistList &playlists);
-    virtual void showMore(const QString &artist, const QString &album = QString::null);
+    virtual void showMore(const QString &artist, const QString &album = QString());
     virtual void removeTrack(const QString &playlist, const QStringList &files);
 
     virtual QString playlist() const;
@@ -158,7 +161,7 @@ public:
     class ActionHandler;
 
 protected:
-    virtual QWidgetStack *playlistStack() const;
+    virtual QStackedWidget *playlistStack() const;
     virtual void setupPlaylist(Playlist *playlist, const QString &iconName);
     virtual void removePlaylist(Playlist *playlist) = 0;
 
@@ -166,7 +169,7 @@ protected:
     bool containsPlaylistFile(const QString &file) const;
 
     QString playlistNameDialog(const QString &caption = i18n("Create New Playlist"),
-                               const QString &suggest = QString::null,
+                               const QString &suggest = QString(),
                                bool forceUnique = true) const;
     QString uniquePlaylistName(const QString &suggest = i18n("Playlist")) const;
 
@@ -181,7 +184,7 @@ private:
     void readConfig();
     void saveConfig();
 
-    QWidgetStack     *m_playlistStack;
+    QStackedWidget   *m_playlistStack;
     HistoryPlaylist  *m_historyPlaylist;
     UpcomingPlaylist *m_upcomingPlaylist;
     ActionHandler    *m_actionHandler;
@@ -194,12 +197,14 @@ private:
     bool        m_searchEnabled;
     bool        m_playing;
 
-    QGuardedPtr<SearchPlaylist> m_showMorePlaylist;
-    QGuardedPtr<Playlist> m_belowShowMorePlaylist;
-    QGuardedPtr<DynamicPlaylist> m_dynamicPlaylist;
-    QGuardedPtr<Playlist> m_belowDistraction;
+    QPointer<SearchPlaylist> m_showMorePlaylist;
+    QPointer<Playlist> m_belowShowMorePlaylist;
+    QPointer<DynamicPlaylist> m_dynamicPlaylist;
+    QPointer<Playlist> m_belowDistraction;
 
     QWidget *m_distraction;
+
+    static PlaylistCollection *m_instance;
 };
 
 /**
@@ -217,7 +222,7 @@ private:
     KAction *createAction(const QString &text,
                           const char *slot,
                           const char *name,
-                          const QString &icon = QString::null,
+                          const QString &icon = QString(),
                           const KShortcut &shortcut = KShortcut());
 private slots:
     void slotPlayFirst()     { m_collection->playFirst(); }
@@ -268,3 +273,5 @@ private:
 };
 
 #endif
+
+// vim: set et sw=4 tw=0 sta:
