@@ -4,6 +4,9 @@
 
     copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
+
+    copyright            : (C) 2004 - 2008 by Michael Pyne
+    email                : michael.pyne@kdemail.net
  ***************************************************************************/
 
 /***************************************************************************
@@ -15,29 +18,40 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "systemtray.h"
+
 #include <klocale.h>
 #include <kiconloader.h>
-#include <kpassivepopup.h>
 #include <kiconeffect.h>
 #include <kaction.h>
-#include <kpopupmenu.h>
-#include <kglobalsettings.h>
 #include <kdebug.h>
+#include <kactioncollection.h>
+#include <kactionmenu.h>
+#include <ktoggleaction.h>
+#include <kvbox.h>
 
-#include <qvbox.h>
-#include <qtimer.h>
-#include <qcolor.h>
-#include <qpushbutton.h>
-#include <qtooltip.h>
-#include <qpainter.h>
-#include <qvaluevector.h>
-#include <qstylesheet.h>
-#include <qpalette.h>
+#include <QTimer>
+#include <QColor>
+#include <QPushButton>
+#include <QPalette>
+#include <QWheelEvent>
+#include <QPixmap>
+#include <QMenu>
+#include <Q3MimeSourceFactory>
+#include <QLabel>
+#include <QPainter>
+#include <QVBoxLayout>
+#include <QMouseEvent>
+#include <QDesktopWidget>
+#include <QApplication>
+#include <QTextDocument> // Qt::escape()
 
+#ifdef Q_WS_X11
 #include <netwm.h>
+#include <QX11Info>
+#endif
 
 #include "tag.h"
-#include "systemtray.h"
 #include "actioncollection.h"
 #include "playermanager.h"
 #include "collectionlist.h"
@@ -45,70 +59,47 @@
 
 using namespace ActionCollection;
 
-static bool copyImage(QImage &dest, QImage &src, int x, int y);
-
-class FlickerFreeLabel : public QLabel
+PassiveInfo::PassiveInfo(QSystemTrayIcon *parent) :
+    QFrame(static_cast<QWidget *>(0), 
+        Qt::ToolTip | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint
+    ),
+    m_icon(parent),
+    m_timer(new QTimer),
+    m_layout(new QVBoxLayout(this)),
+    m_view(0),
+    m_justDie(false)
 {
-public:
-    FlickerFreeLabel(const QString &text, QWidget *parent, const char *name = 0) :
-        QLabel(text, parent, name)
-    {
-        m_textColor = paletteForegroundColor();
-        m_bgColor = parentWidget()->paletteBackgroundColor();
-        setBackgroundMode(NoBackground);
-    }
-
-    QColor textColor() const
-    {
-        return m_textColor;
-    }
-
-    QColor backgroundColor() const
-    {
-        return m_bgColor;
-    }
-
-protected:
-    virtual void drawContents(QPainter *p)
-    {
-        // We want to intercept the drawContents call and draw on a pixmap
-        // instead of the window to keep flicker to an absolute minimum.
-        // Since Qt doesn't refresh the background, we need to do so
-        // ourselves.
-
-        QPixmap pix(size());
-        QPainter pixPainter(&pix);
-
-        pixPainter.fillRect(rect(), m_bgColor);
-        QLabel::drawContents(&pixPainter);
-
-        bitBlt(p->device(), QPoint(0, 0), &pix, rect(), CopyROP);
-    }
-
-    private:
-    QColor m_textColor;
-    QColor m_bgColor;
-};
-
-PassiveInfo::PassiveInfo(QWidget *parent, const char *name) :
-    KPassivePopup(parent, name), m_timer(new QTimer), m_justDie(false)
-{
-    // I'm so sick and tired of KPassivePopup screwing this up
-    // that I'll just handle the timeout myself, thank you very much.
-    KPassivePopup::setTimeout(0);
-
     connect(m_timer, SIGNAL(timeout()), SLOT(timerExpired()));
+    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+
+    // I'd like StyledPanel but it doesn't work in the default KDE style...
+    setFrameStyle(Box | Plain);
+    setLineWidth(3);
 }
 
-void PassiveInfo::setTimeout(int delay)
+void PassiveInfo::startTimer(int delay)
 {
-    m_timer->changeInterval(delay);
+    m_timer->start(delay);
 }
 
 void PassiveInfo::show()
 {
-    KPassivePopup::show();
     m_timer->start(3500);
+    setWindowOpacity(1.0);
+    QFrame::show();
+}
+
+QWidget *PassiveInfo::view() const
+{
+    return m_view;
+}
+
+void PassiveInfo::setView(QWidget *view)
+{
+    m_layout->addWidget(view);
+    view->show(); // We are still hidden though.
+    adjustSize();
+    positionSelf();
 }
 
 void PassiveInfo::timerExpired()
@@ -133,64 +124,114 @@ void PassiveInfo::leaveEvent(QEvent *)
     m_timer->start(50);
 }
 
+void PassiveInfo::hideEvent(QHideEvent *)
+{
+}
+
+void PassiveInfo::positionSelf()
+{
+    // Start with a QRect of our size, move it to the right spot.
+    QRect r(rect());
+    QPoint iconCenter(m_icon->geometry().center());
+    QRect iconRect(m_icon->geometry());
+    QRect curScreen(QApplication::desktop()->availableGeometry(iconCenter));
+
+    // Decide if going on left or right.  We will be somewhere in availableGeometry, we want
+    // to be on the side that opposite the systray icon.  i.e. if icon on right side of
+    // screen, we go left and have our right side flush with the left edge.
+    bool onRight = iconCenter.x() < curScreen.center().x();
+
+    // For top or bottom, we want to be near the systray icon.  Assume that the systray icon
+    // is near an edge and therefore it's on the top if its y() is less than the center.
+    bool onTop = iconCenter.y() < curScreen.center().y();
+
+    QPoint anchor;
+
+    if(onRight)
+        anchor.setX(iconRect.right());
+    else
+        anchor.setX(iconRect.left());
+
+    if(onTop)
+        anchor.setY(qMax(iconRect.bottom(), curScreen.top()));
+    else
+        anchor.setY(qMax(iconRect.top(), curScreen.bottom()));
+
+    // Now make our rect hit that anchor.
+    if(onTop && onRight)
+        r.moveTopLeft(anchor);
+    else if(onTop && !onRight)
+        r.moveTopRight(anchor);
+    else if(!onTop && onRight)
+        r.moveBottomLeft(anchor);
+    else
+        r.moveBottomRight(anchor);
+
+    move(r.topLeft());
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-SystemTray::SystemTray(QWidget *parent, const char *name) : KSystemTray(parent, name),
-                                                            m_popup(0),
-                                                            m_fadeTimer(0),
-                                                            m_fade(true)
+SystemTray::SystemTray(QWidget *parent) : KSystemTrayIcon(parent),
+                                          m_popup(0),
+                                          m_fadeTimer(0),
+                                          m_fade(true),
+                                          m_hasCompositionManager(false)
 
 {
     // This should be initialized to the number of labels that are used.
+    m_labels.fill(0, 3);
 
-    m_labels.reserve(3);
+    m_appPix = loadIcon("juk");
 
-    m_appPix = loadIcon("juk_dock");
+    m_playPix = createPixmap("media-playback-start");
+    m_pausePix = createPixmap("media-playback-pause");
 
-    m_playPix = createPixmap("player_play");
-    m_pausePix = createPixmap("player_pause");
+    m_forwardPix = SmallIcon("media-skip-forward");
+    m_backPix = SmallIcon("media-skip-backward");
 
-    m_forwardPix = loadIcon("player_end");
-    m_backPix = loadIcon("player_start");
-
-    setPixmap(m_appPix);
+    setIcon(m_appPix);
 
     setToolTip();
 
     // Just create this here so that it show up in the DCOP interface and the key
     // bindings dialog.
 
-    new KAction(i18n("Redisplay Popup"), KShortcut(), this,
-                SLOT(slotPlay()), actions(), "showPopup");
-    
-    KPopupMenu *cm = contextMenu();
+    KAction *rpaction = new KAction(i18n("Redisplay Popup"), this);
+    ActionCollection::actions()->addAction("showPopup", rpaction);
+    connect(rpaction, SIGNAL(triggered(bool) ), SLOT(slotPlay()));
+
+    QMenu *cm = contextMenu();
 
     connect(PlayerManager::instance(), SIGNAL(signalPlay()), this, SLOT(slotPlay()));
     connect(PlayerManager::instance(), SIGNAL(signalPause()), this, SLOT(slotPause()));
     connect(PlayerManager::instance(), SIGNAL(signalStop()), this, SLOT(slotStop()));
 
-    action("play")->plug(cm);
-    action("pause")->plug(cm);
-    action("stop")->plug(cm);
-    action("forward")->plug(cm);
-    action("back")->plug(cm);
+    cm->addAction( action("play") );
+    cm->addAction( action("pause") );
+    cm->addAction( action("stop") );
+    cm->addAction( action("forward") );
+    cm->addAction( action("back") );
 
-    cm->insertSeparator();
+    cm->addSeparator();
 
     // Pity the actionCollection doesn't keep track of what sub-menus it has.
 
     KActionMenu *menu = new KActionMenu(i18n("&Random Play"), this);
-    menu->insert(action("disableRandomPlay"));
-    menu->insert(action("randomPlay"));
-    menu->insert(action("albumRandomPlay"));
-    menu->plug(cm);
+    actionCollection()->addAction("randomplay", menu);
+    menu->addAction(action("disableRandomPlay"));
+    menu->addAction(action("randomPlay"));
+    menu->addAction(action("albumRandomPlay"));
+    cm->addAction( menu );
 
-    action("togglePopups")->plug(cm);
+    cm->addAction( action("togglePopups") );
 
-    m_fadeTimer = new QTimer(this, "systrayFadeTimer");
+    m_fadeTimer = new QTimer(this);
+    m_fadeTimer->setObjectName("systrayFadeTimer");
     connect(m_fadeTimer, SIGNAL(timeout()), SLOT(slotNextStep()));
+    connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(slotActivated(QSystemTrayIcon::ActivationReason)));
 
     if(PlayerManager::instance()->playing())
         slotPlay();
@@ -214,7 +255,7 @@ void SystemTray::slotPlay()
 
     QPixmap cover = PlayerManager::instance()->playingFile().coverInfo()->pixmap(CoverInfo::Thumbnail);
 
-    setPixmap(m_playPix);
+    setIcon(m_playPix);
     setToolTip(PlayerManager::instance()->playingString(), cover);
     createPopup();
 }
@@ -222,7 +263,7 @@ void SystemTray::slotPlay()
 void SystemTray::slotTogglePopup()
 {
     if(m_popup && m_popup->view()->isVisible())
-        m_popup->setTimeout(50);
+        m_popup->startTimer(50);
     else
         slotPlay();
 }
@@ -238,45 +279,60 @@ void  SystemTray::slotPopupLargeCover()
 
 void SystemTray::slotStop()
 {
-    setPixmap(m_appPix);
+    setIcon(m_appPix);
     setToolTip();
 
     delete m_popup;
     m_popup = 0;
+    m_fadeTimer->stop();
 }
 
 void SystemTray::slotPopupDestroyed()
 {
-    for(unsigned i = 0; i < m_labels.capacity(); ++i)
+    for(int i = 0; i < m_labels.size(); ++i)
         m_labels[i] = 0;
 }
 
 void SystemTray::slotNextStep()
 {
-    QColor result;
+    // Could happen I guess if the timeout event were queued while we're deleting m_popup
+    if(!m_popup)
+        return;
 
     ++m_step;
 
-    // If we're not fading, immediately show the labels
-    if(!m_fade)
-        m_step = STEPS;
-
-    result = interpolateColor(m_step);
-
-    for(unsigned i = 0; i < m_labels.capacity() && m_labels[i]; ++i)
-        m_labels[i]->setPaletteForegroundColor(result);
-
-    if(m_step == STEPS) {
+    // If we're not fading, immediately stop the fadeout
+    if(!m_fade || m_step == STEPS) {
         m_step = 0;
         m_fadeTimer->stop();
         emit fadeDone();
+        return;
+    }
+
+    if(m_hasCompositionManager) {
+        m_popup->setWindowOpacity((1.0 * STEPS - m_step) / STEPS);
+    }
+    else {
+        QColor result = interpolateColor(m_step);
+
+        for(int i = 0; i < m_labels.size() && m_labels[i]; ++i) {
+            QPalette palette;
+            palette.setColor(m_labels[i]->foregroundRole(), result);
+            m_labels[i]->setPalette(palette);
+        }
     }
 }
 
 void SystemTray::slotFadeOut()
 {
-    m_startColor = m_labels[0]->textColor();
-    m_endColor = m_labels[0]->backgroundColor();
+    m_startColor = m_labels[0]->palette().color( QPalette::Text ); //textColor();
+    m_endColor = m_labels[0]->palette().color( QPalette::Window ); //backgroundColor();
+
+#ifdef Q_WS_X11
+    m_hasCompositionManager = QX11Info::isCompositingManagerRunning();
+#else
+    m_hasCompositionManager = true; // Assume yes for non-X11
+#endif
 
     connect(this, SIGNAL(fadeDone()), m_popup, SLOT(hide()));
     connect(m_popup, SIGNAL(mouseEntered()), this, SLOT(slotMouseInPopup()));
@@ -288,9 +344,12 @@ void SystemTray::slotFadeOut()
 // don't have to do it ourselves.
 void SystemTray::slotMouseInPopup()
 {
-    m_endColor = m_labels[0]->textColor();
+    m_endColor = m_labels[0]->palette().color( QPalette::Text ); //textColor();
     disconnect(SIGNAL(fadeDone()));
 
+    if(m_hasCompositionManager)
+        m_popup->setWindowOpacity(1.0);
+
     m_step = STEPS - 1; // Simulate end of fade to solid text
     slotNextStep();
 }
@@ -299,9 +358,9 @@ void SystemTray::slotMouseInPopup()
 // private methods
 ////////////////////////////////////////////////////////////////////////////////
 
-QVBox *SystemTray::createPopupLayout(QWidget *parent, const FileHandle &file)
+KVBox *SystemTray::createPopupLayout(QWidget *parent, const FileHandle &file)
 {
-    QVBox *infoBox = 0;
+    KVBox *infoBox = 0;
 
     if(buttonsToLeft()) {
 
@@ -310,7 +369,7 @@ QVBox *SystemTray::createPopupLayout(QWidget *parent, const FileHandle &file)
         createButtonBox(parent);
         addSeparatorLine(parent);
 
-        infoBox = new QVBox(parent);
+        infoBox = new KVBox(parent);
 
         // Another line, and the cover, if there's a cover, and if
         // it's selected to be shown
@@ -329,7 +388,7 @@ QVBox *SystemTray::createPopupLayout(QWidget *parent, const FileHandle &file)
             addSeparatorLine(parent);
         }
 
-        infoBox = new QVBox(parent);
+        infoBox = new KVBox(parent);
 
         addSeparatorLine(parent);
         createButtonBox(parent);
@@ -344,10 +403,10 @@ void SystemTray::createPopup()
 {
     FileHandle playingFile = PlayerManager::instance()->playingFile();
     Tag *playingInfo = playingFile.tag();
-    
+
     // If the action exists and it's checked, do our stuff
 
-    if(!action<KToggleAction>("togglePopups")->isChecked())
+    if(!ActionCollection::action<KToggleAction>("togglePopups")->isChecked())
         return;
 
     delete m_popup;
@@ -363,112 +422,100 @@ void SystemTray::createPopup()
     connect(m_popup, SIGNAL(destroyed()), SLOT(slotPopupDestroyed()));
     connect(m_popup, SIGNAL(timeExpired()), SLOT(slotFadeOut()));
 
-    QHBox *box = new QHBox(m_popup, "popupMainLayout");
+    KHBox *box = new KHBox(m_popup);
     box->setSpacing(15); // Add space between text and buttons
 
-    QVBox *infoBox = createPopupLayout(box, playingFile);
+    KVBox *infoBox = createPopupLayout(box, playingFile);
 
-    for(unsigned i = 0; i < m_labels.capacity(); ++i) {
-        m_labels[i] = new FlickerFreeLabel(" ", infoBox);
-        m_labels[i]->setAlignment(AlignRight | AlignVCenter);
+    for(int i = 0; i < m_labels.size(); ++i) {
+        QLabel *l = new QLabel(" ", infoBox);
+        l->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+        m_labels[i] = l;
     }
 
-    // We don't want an autodelete popup.  There are times when it will need
-    // to be hidden before the timeout.
-
-    m_popup->setAutoDelete(false);
-
     // We have to set the text of the labels after all of the
     // widgets have been added in order for the width to be calculated
     // correctly.
 
     int labelCount = 0;
 
-    QString title = QStyleSheet::escape(playingInfo->title());
-    m_labels[labelCount++]->setText(QString("<qt><nobr><h2>%1</h2></nobr><qt>").arg(title));
+    QString title = Qt::escape(playingInfo->title());
+    m_labels[labelCount++]->setText(QString("<qt><nobr><h2>%1</h2></nobr></qt>").arg(title));
 
     if(!playingInfo->artist().isEmpty())
         m_labels[labelCount++]->setText(playingInfo->artist());
 
     if(!playingInfo->album().isEmpty()) {
-        QString album = QStyleSheet::escape(playingInfo->album());
+        QString album = Qt::escape(playingInfo->album());
         QString s = playingInfo->year() > 0
             ? QString("<qt><nobr>%1 (%2)</nobr></qt>").arg(album).arg(playingInfo->year())
             : QString("<qt><nobr>%1</nobr></qt>").arg(album);
         m_labels[labelCount++]->setText(s);
     }
 
-    m_startColor = m_labels[0]->backgroundColor();
-    m_endColor = m_labels[0]->textColor();
-
-    slotNextStep();
-    m_fadeTimer->start(1500 / STEPS);
-
     m_popup->setView(box);
     m_popup->show();
 }
 
 bool SystemTray::buttonsToLeft() const
 {
-    // The following code was nicked from kpassivepopup.cpp
+    QPoint iconCenter(geometry().center());
+    QRect curScreen(QApplication::desktop()->availableGeometry(iconCenter));
 
-    NETWinInfo ni(qt_xdisplay(), winId(), qt_xrootwin(), 
-                  NET::WMIconGeometry | NET::WMKDESystemTrayWinFor);
-    NETRect frame, win;
-    ni.kdeGeometry(frame, win);
-    
-    QRect bounds = KGlobalSettings::desktopGeometry(QPoint(win.pos.x, win.pos.y));
-
-    // This seems to accurately guess what side of the icon that
-    // KPassivePopup will popup on.
-    return(win.pos.x < bounds.center().x());
+    // See PassivePopup::positionSelf()
+    return iconCenter.x() < curScreen.center().x();
 }
 
 QPixmap SystemTray::createPixmap(const QString &pixName)
 {
-    QPixmap bgPix = m_appPix;
-    QPixmap fgPix = SmallIcon(pixName);
-
-    QImage bgImage = bgPix.convertToImage(); // Probably 22x22
-    QImage fgImage = fgPix.convertToImage(); // Should be 16x16
+    QImage bgImage = m_appPix.pixmap(22).toImage(); // 22x22
+    QImage fgImage = SmallIcon(pixName).toImage(); // Should be 16x16
 
     KIconEffect::semiTransparent(bgImage);
-    copyImage(bgImage, fgImage, (bgImage.width() - fgImage.width()) / 2,
-              (bgImage.height() - fgImage.height()) / 2);
 
-    bgPix.convertFromImage(bgImage);
-    return bgPix;
+    // Center smaller image inside larger one.
+    QRect rect = fgImage.rect();
+    rect.moveCenter(bgImage.rect().center());
+
+    // Must use begin()/end() (or scope in this case) to avoid returning a
+    // QPixmap constructed from a QImage being painted on.
+    {
+        QPainter painter(&bgImage);
+        painter.drawImage(rect.topLeft(), fgImage);
+    }
+
+    return QPixmap::fromImage(bgImage);
 }
 
 void SystemTray::createButtonBox(QWidget *parent)
 {
-    QVBox *buttonBox = new QVBox(parent);
-    
+    KVBox *buttonBox = new KVBox(parent);
+
     buttonBox->setSpacing(3);
 
-    QPushButton *forwardButton = new QPushButton(m_forwardPix, 0, buttonBox, "popup_forward");
-    forwardButton->setFlat(true);
+    QPushButton *forwardButton = new QPushButton(m_forwardPix, 0, buttonBox);
+    forwardButton->setObjectName("popup_forward");
     connect(forwardButton, SIGNAL(clicked()), SLOT(slotForward()));
 
-    QPushButton *backButton = new QPushButton(m_backPix, 0, buttonBox, "popup_back");
-    backButton->setFlat(true);
+    QPushButton *backButton = new QPushButton(m_backPix, 0, buttonBox);
+    backButton->setObjectName("popup_back");
     connect(backButton, SIGNAL(clicked()), SLOT(slotBack()));
 }
 
 /**
- * What happens here is that the action->activate() call will end up invoking
+ * What happens here is that the action->trigger() call will end up invoking
  * createPopup(), which sets m_fade to true.  Before the text starts fading
  * control returns to this function, which resets m_fade to false.
  */
 void SystemTray::slotBack()
 {
-    action("back")->activate();
+    action("back")->trigger();
     m_fade = false;
 }
 
 void SystemTray::slotForward()
 {
-    action("forward")->activate();
+    action("forward")->trigger();
     m_fade = false;
 }
 
@@ -487,7 +534,8 @@ void SystemTray::addCoverButton(QWidget *parent, const QPixmap &cover)
 {
     QPushButton *coverButton = new QPushButton(parent);
 
-    coverButton->setPixmap(cover);
+    coverButton->setIconSize(cover.size());
+    coverButton->setIcon(cover);
     coverButton->setFixedSize(cover.size());
     coverButton->setFlat(true);
 
@@ -513,34 +561,37 @@ QColor SystemTray::interpolateColor(int step, int steps)
 
 void SystemTray::setToolTip(const QString &tip, const QPixmap &cover)
 {
-    QToolTip::remove(this);
-
-    if(tip.isNull())
-        QToolTip::add(this, i18n("JuK"));
+    if(tip.isEmpty())
+        KSystemTrayIcon::setToolTip(i18n("JuK"));
     else {
         QPixmap myCover = cover;
         if(cover.isNull())
             myCover = DesktopIcon("juk");
 
-        QImage coverImage = myCover.convertToImage();
-        if(coverImage.size().width() > 32 || coverImage.size().height() > 32)
-            coverImage = coverImage.smoothScale(32, 32);
+        QImage coverImage = myCover.toImage();
+        QSize iconSize(IconSize(KIconLoader::Desktop), IconSize(KIconLoader::Desktop));
+        QSize newIconSize = coverImage.size();
+        newIconSize.scale(iconSize, Qt::KeepAspectRatio);
 
-        QMimeSourceFactory::defaultFactory()->setImage("tipCover", coverImage);
+        if(newIconSize != iconSize)
+            coverImage = coverImage.scaled(newIconSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 
-        QString html = i18n("%1 is Cover Art, %2 is the playing track, %3 is the appname",
-                            "<center><table cellspacing=\"2\"><tr><td valign=\"middle\">%1</td>"
-                            "<td valign=\"middle\">%2</td></tr></table><em>%3</em></center>");
-        html = html.arg("<img valign=\"middle\" src=\"tipCover\"");
-        html = html.arg(QString("<nobr>%1</nobr>").arg(tip), i18n("JuK"));
+        Q3MimeSourceFactory::defaultFactory()->setImage("tipCover", coverImage);
 
-        QToolTip::add(this, html);
+        QString html = i18nc("%1 is Cover Art, %2 is the playing track, %3 is the appname",
+                            "<table style=\"font-size:14pt;\" cellspacing=\"2em\">"
+                            "<tr><td rowspan=\"2\">%1</td><td align=\"center\" valign=\"middle\">%2</td></tr>"
+                            "<tr><td align=\"center\" valign=\"middle\"><em>%3</em></td></tr></table>",
+                            QString("<img valign=\"middle\" src=\"tipCover\">"),
+                            QString("<nobr>%1</nobr>").arg(tip), i18n("JuK"));
+
+        KSystemTrayIcon::setToolTip( html);
     }
 }
 
 void SystemTray::wheelEvent(QWheelEvent *e)
 {
-    if(e->orientation() == Horizontal)
+    if(e->orientation() == Qt::Horizontal)
         return;
 
     // I already know the type here, but this file doesn't (and I don't want it
@@ -548,18 +599,18 @@ void SystemTray::wheelEvent(QWheelEvent *e)
     // that a reinterpret_cast isn't portable when combined with multiple
     // inheritance.  (This is why I don't check the result.)
 
-    switch(e->state()) {
-    case ShiftButton:
+    switch(e->modifiers()) {
+    case Qt::ShiftButton:
         if(e->delta() > 0)
-            action("volumeUp")->activate();
+            action("volumeUp")->trigger();
         else
-            action("volumeDown")->activate();
+            action("volumeDown")->trigger();
         break;
     default:
         if(e->delta() > 0)
-            action("forward")->activate();
+            action("forward")->trigger();
         else
-            action("back")->activate();
+            action("back")->trigger();
         break;
     }
     e->accept();
@@ -568,72 +619,17 @@ void SystemTray::wheelEvent(QWheelEvent *e)
 /*
  * Reimplemented this in order to use the middle mouse button
  */
-void SystemTray::mousePressEvent(QMouseEvent *e)
+void SystemTray::slotActivated(QSystemTrayIcon::ActivationReason reason)
 {
-    switch(e->button()) {
-    case LeftButton:
-    case RightButton:
-    default:
-        KSystemTray::mousePressEvent(e);
-        break;
-    case MidButton:
-        if(!rect().contains(e->pos()))
-            return;
-        if(action("pause")->isEnabled())
-            action("pause")->activate();
-        else
-            action("play")->activate();
-        break;
-    }
+    if (reason != MiddleClick)
+        return;
+
+    if(action("pause")->isEnabled())
+        action("pause")->trigger();
+    else
+        action("play")->trigger();
 }
 
-/*
- * This function copies the entirety of src into dest, starting in
- * dest at x and y.  This function exists because I was unable to find
- * a function like it in either QImage or kdefx
- */
-
-static bool copyImage(QImage &dest, QImage &src, int x, int y)
-{
-    if(dest.depth() != src.depth())
-        return false;
-    if((x + src.width()) >= dest.width())
-        return false;
-    if((y + src.height()) >= dest.height())
-        return false;
-
-    // We want to use KIconEffect::overlay to do this, since it handles
-    // alpha, but the images need to be the same size.  We can handle that.
-
-    QImage large_src(dest);
-
-    // It would perhaps be better to create large_src based on a size, but
-    // this is the easiest way to make a new image with the same depth, size,
-    // etc.
-
-    large_src.detach();
-
-    // However, we do have to specifically ensure that setAlphaBuffer is set
-    // to false
-
-    large_src.setAlphaBuffer(false);
-    large_src.fill(0); // All transparent pixels
-    large_src.setAlphaBuffer(true);
-
-    int w = src.width();
-    int h = src.height();
-    for(int dx = 0; dx < w; dx++)
-        for(int dy = 0; dy < h; dy++)
-            large_src.setPixel(dx + x, dy + y, src.pixel(dx, dy));
-
-    // Apply effect to image
-
-    KIconEffect::overlay(dest, large_src);
-
-    return true;
-}
-
-
 #include "systemtray.moc"
 
-// vim: et sw=4 ts=8
+// vim: set et sw=4 tw=0 sta:
